#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

int main(int argc, char** argv) {
    if (argc < 3) {
        printf("Usage: cafromutil <input file path> <output file path>\n");
        return 1;
    }
    char *inFilePath = argv[1];
    char *outFilePath = argv[2];
    FILE *inFp;
    FILE *outFp;
    size_t inputFileSize;
    size_t outputFileSize;
    char *inputBuffer;
    char *outputBuffer;
    
    char sectionHeader[5];
    char *paktData = NULL;
    char *audioData = NULL;
    int64_t paktSize;
    int64_t audioSize;
    
    inFp = fopen(inFilePath, "rb");
    if (inFp == NULL) {
        fprintf(stderr, "Could not open input file.\n");
        return 2;
    }
    fseek(inFp, 0, SEEK_END);
    inputFileSize = ftell(inFp);
    fseek(inFp, 0, SEEK_SET);
    inputBuffer = malloc(inputFileSize);
    fread(inputBuffer, 1, inputFileSize, inFp);
    fclose(inFp);
    
    int pos = 0;
    char secSizeA[8];
    int64_t secSize;
    
    for (int i=0; i<inputFileSize - 4; i++) {
        if ((inputBuffer[i] >= 0x61 && inputBuffer[i] <= 0x7A) && (inputBuffer[i+1] >= 0x61 && inputBuffer[i+1] <= 0x7A) && (inputBuffer[i+2] >= 0x61 && inputBuffer[i+2] <= 0x7A) && (inputBuffer[i+3] >= 0x61 && inputBuffer[i+3] <= 0x7A)) {
            sectionHeader[0] = inputBuffer[i];
            sectionHeader[1] = inputBuffer[i + 1];
            sectionHeader[2] = inputBuffer[i + 2];
            sectionHeader[3] = inputBuffer[i + 3];
            sectionHeader[4] = '\0';
            if ((strcmp(sectionHeader, "pakt") == 0) || (strcmp(sectionHeader, "data") == 0)) {
                pos = i + 4;
                
                //Convert from little endian to big endian
                secSizeA[7] = inputBuffer[pos];
                secSizeA[6] = inputBuffer[pos + 1];
                secSizeA[5] = inputBuffer[pos + 2];
                secSizeA[4] = inputBuffer[pos + 3];
                secSizeA[3] = inputBuffer[pos + 4];
                secSizeA[2] = inputBuffer[pos + 5];
                secSizeA[1] = inputBuffer[pos + 6];
                secSizeA[0] = inputBuffer[pos + 7];
                secSize = *(int64_t*)(&secSizeA[0]);
                
                if (strcmp(sectionHeader, "pakt") == 0) {
                    printf("pakt section size: %lld bytes\n", secSize);
                    paktSize = secSize;
                    paktData = malloc(secSize);
                    memcpy(paktData, &inputBuffer[pos + 8], secSize);
                    
                }
                else if (strcmp(sectionHeader, "data") == 0) {
                    if (secSize < 0) {
                        secSize = inputFileSize - (pos + 12);
                    }
                    audioSize = secSize - 4;
                    printf("Audio data section size: %lld bytes\n", audioSize);
                    audioData = malloc(audioSize);
                    memcpy(audioData, &inputBuffer[pos + 12], audioSize); //Copy audio data, skip 4 bytes reserved for edit count.
                }
                memset(sectionHeader, 0, sizeof(sectionHeader));
            }
        }
    }
    
    free(inputBuffer);
    
    if (paktData == NULL) {
        fprintf(stderr, "Could not locate pakt data in file.\n");
        return 3;
    } else if (audioData == NULL) {
        fprintf(stderr, "Could not locate audio data in file.\n");
        return 4;
    }
    
    char header[12] = {0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x2C};
    
    //File structure: <header> <4th set of 4 bytes of pakt data> <audio data>
    outputFileSize = 12 + 4 + audioSize;
    printf("Output file size: %zu bytes\n", outputFileSize);
    outputBuffer = malloc(outputFileSize);
    memcpy(outputBuffer, header, 12);
    memcpy(&outputBuffer[12], &paktData[12], 4);
    memcpy(&outputBuffer[16], audioData, audioSize);
    
    outFp = fopen(outFilePath, "wb");
    if (outFp == NULL) {
        fprintf(stderr, "Could not create file.\n");
        return 5;
    }
    
    if (fwrite(outputBuffer, 1, outputFileSize, outFp) != outputFileSize) {
        fprintf(stderr, "Could not write to output file.\n");
        return 6;
    }
    fclose(outFp);
    free(paktData);
    free(audioData);
    free(outputBuffer);
    
    printf("Converted file successfully, saved to: %s\n", outFilePath);
    
    return 0;
}
